@@ -1,3 +1,4 @@
+import copy
 import re
 import string
 import sys
@@ -6,7 +7,7 @@ import warnings
 import numpy as np
 
 from pyfits.util import _str_to_num, _is_int, deprecated, maketrans, translate
-from pyfits.verify import _Verify, _ErrList
+from pyfits.verify import _Verify, _ErrList, VerifyError
 
 
 __all__ = ['Card', 'CardList', 'RecordValuedKeywordCard', 'create_card',
@@ -1317,6 +1318,11 @@ class CardList(list):
         """Format a list of cards into a printable string."""
         return '\n'.join(map(str, self))
 
+    def __deepcopy__(self, memo):
+        dup = CardList([copy.deepcopy(c, memo) for c in self])
+        memo[id(self)] = dup
+        return dup
+
     def _get_mod(self):
         mod = self.__dict__.get('_mod', False)
         if not mod:
