@@ -1,8 +1,6 @@
 import gzip
 import os
-import signal
 import sys
-import threading
 import warnings
 
 import numpy as np
@@ -20,12 +18,13 @@ from pyfits.hdu.image import PrimaryHDU, ImageHDU
 from pyfits.hdu.table import _TableBaseHDU
 from pyfits.util import (Extendable, _is_int, _tmp_name, _with_extensions,
                          _pad_length, BLOCK_SIZE, isfile, fileobj_name,
-                         fileobj_closed, fileobj_mode)
-from pyfits.verify import _Verify, _ErrList
+                         fileobj_closed, fileobj_mode, ignore_sigint,
+                         _get_array_memmap, indent)
+from pyfits.verify import _Verify, _ErrList, VerifyError
 
 
 @_with_extensions
-def fitsopen(name, mode="copyonwrite", memmap=False, classExtensions={},
+def fitsopen(name, mode='readonly', memmap=None, classExtensions={},
              **kwargs):
     """Factory function to open a FITS file and return an `HDUList` object.
 
@@ -35,12 +34,13 @@ def fitsopen(name, mode="copyonwrite", memmap=False, classExtensions={},
         File to be opened.
 
     mode : str
-        Open mode, 'copyonwrite' (default), 'readonly', 'update',
-        'append', or 'ostream'.
+        Open mode, 'readonly' (default), 'update', 'append', 'denywrite', or
+        'ostream'.
 
         If `name` is a file object that is already opened, `mode` must
         match the mode the file was opened with, copyonwrite (rb),
-        readonly (rb), update (rb+), append (ab+), ostream (w)).
+        readonly (rb), update (rb+), append (ab+), ostream (w),
+        denywrite (rb)).
 
     memmap : bool
         Is memory mapping to be used?
@@ -139,8 +139,11 @@ class HDUList(list, _Verify):
             if not isinstance(hdu, _BaseHDU):
                 raise TypeError(
                       "Element %d in the HDUList input is not an HDU." % idx)
+
         super(HDUList, self).__init__(hdus)
 
+        self.update_extend()
+
     def __iter__(self):
         for idx in range(len(self)):
             yield self[idx]
@@ -252,13 +255,13 @@ class HDUList(list, _Verify):
                     break
                 # check in the case there is extra space after the last HDU or
                 # corrupted HDU
-                except ValueError, err:
+                except (VerifyError, ValueError), err:
                     warnings.warn(
-                        'Required keywords missing when trying to read '
-                        'HDU #%d (note: PyFITS uses zero-based indexing.\n'
-                        '          %s\n          There may be extra '
-                        'bytes after the last HDU or the file is corrupted.' %
-                        (len(hdulist), err))
+                        'Error validating header for HDU #%d (note: PyFITS '
+                        'uses zero-based indexing).\n%s\n'
+                        'There may be extra bytes after the last HDU or the '
+                        'file is corrupted.' %
+                        (len(hdulist), indent(str(err))))
                     break
                 except IOError, err:
                     if ffo.writeonly:
@@ -268,13 +271,10 @@ class HDUList(list, _Verify):
 
             # If we're trying to read only and no header units were found,
             # raise and exception
-            if mode == 'readonly' and len(hdulist) == 0:
-                raise IOError('Empty FITS file')
+            if mode in ('readonly', 'denywrite') and len(hdulist) == 0:
+                raise IOError('Empty or corrupt FITS file')
 
             # initialize/reset attributes to be used in "update/append" mode
-            # CardList needs its own _mod attribute since it has methods to change
-            # the content of header without being able to pass it to the header
-            # object
             hdulist._resize = False
             hdulist._truncate = False
 
@@ -311,7 +311,7 @@ class HDUList(list, _Verify):
             file       File object associated with the HDU
             filename   Name of associated file object
             filemode   Mode in which the file was opened (readonly, copyonwrite,
-                       update, append, ostream)
+                       update, append, denywrite, ostream)
             resized    Flag that when `True` indicates that the data has been
                        resized since the last read/write so the returned values
                        may not be valid.
@@ -419,8 +419,7 @@ class HDUList(list, _Verify):
         self._resize = True
         self._truncate = False
         # make sure the EXTEND keyword is in primary HDU if there is extension
-        if len(self) > 1:
-            self.update_extend()
+        self.update_extend()
 
     @_with_extensions
     def append(self, hdu, classExtensions={}):
@@ -473,8 +472,7 @@ class HDUList(list, _Verify):
         self._truncate = False
 
         # make sure the EXTEND keyword is in primary HDU if there is extension
-        if len(self) > 1:
-            self.update_extend()
+        self.update_extend()
 
     def index_of(self, key):
         """
@@ -533,6 +531,7 @@ class HDUList(list, _Verify):
                 continue
 
     @_with_extensions
+    @ignore_sigint
     def flush(self, output_verify='fix', verbose=False,
               classExtensions={}):
         """
@@ -556,22 +555,6 @@ class HDUList(list, _Verify):
             class.
         """
 
-        # Get the name of the current thread and determine if this is a single treaded application
-        curr_thread = threading.currentThread()
-        single_thread = (threading.activeCount() == 1) and \
-                        (curr_thread.getName() == 'MainThread')
-
-        # Define new signal interput handler
-        if single_thread:
-            keyboard_interrupt_sent = False
-            def new_sigint(*args):
-                warnings.warn('KeyboardInterrupt ignored until flush is '
-                              'complete!')
-                keyboard_interrupt_sent = True
-
-            # Install new handler
-            old_handler = signal.signal(signal.SIGINT, new_sigint)
-
         if self.__file.mode not in ('append', 'update', 'ostream'):
             warnings.warn("Flush for '%s' mode is not supported."
                           % self.__file.mode)
@@ -641,8 +624,26 @@ class HDUList(list, _Verify):
                         # TODO: Fix this once new HDU writing API is settled on
                         (hdu._hdrLoc, hdu._datLoc, hdu._datSpan) = \
                                hdu._writeto(hdulist.__file, checksum=checksum)
+
+                    if sys.platform.startswith('win'):
+                        # Collect a list of open mmaps to the data; this well
+                        # be used later.  See below.
+                        mmaps = [(idx, _get_array_memmap(hdu.data), hdu.data)
+                                 for idx, hdu in enumerate(self)
+                                 if hdu._data_loaded]
+
                     hdulist.__file.close()
                     self.__file.close()
+
+                    if sys.platform.startswith('win'):
+                        # Close all open mmaps to the data.  This is only
+                        # necessary on Windows, which will not allow a file to
+                        # be renamed or deleted until all handles to that file
+                        # have been closed.
+                        for idx, map, arr in mmaps:
+                            if map is not None:
+                                map.base.close()
+
                     os.remove(self.__file.name)
 
                     if verbose:
@@ -660,8 +661,35 @@ class HDUList(list, _Verify):
                     ffo = _File(old_file, mode='update', memmap=old_memmap)
 
                     self.__file = ffo
+
+                    for hdu in self:
+                        # Need to update the _file attribute and close any open
+                        # mmaps on each HDU
+                        if (hdu._data_loaded and
+                            _get_array_memmap(hdu.data) is not None):
+                            del hdu.data
+                        hdu._file = ffo
+
                     if verbose:
                         print 'reopen the newly renamed file', old_name
+
+                    if sys.platform.startswith('win'):
+                        # On Windows, all the original data mmaps were closed
+                        # above.  However, it's possible that the user still
+                        # has references to the old data which would no longer
+                        # work (possibly even cause a segfault if they try to
+                        # access it).  This replaces the buffers used by the
+                        # original arrays with the buffers of mmap arrays
+                        # created from the new file.  This seems to work, but
+                        # it's a flaming hack and carries no guarantees that it
+                        # won't lead to odd behavior in practice.  Better to
+                        # just not keep references to data from files that had
+                        # to be resized upon flushing (on Windows--again, this
+                        # is no problem on Linux).
+                        for idx, map, arr in mmaps:
+                            arr.data = self[idx].data.data
+                        del mmaps  # Just to be sure
+
                 else:
                     #
                     # The underlying file is not a file object, it is a file
@@ -727,8 +755,14 @@ class HDUList(list, _Verify):
                                   hdu.name, extver
                     if hdu._data_loaded:
                         if hdu.data is not None:
-                            if isinstance(hdu.data, Memmap):
-                                hdu.data.sync()
+                            # Seek through the array's bases for an memmap'd
+                            # array; we can't rely on the _File object to give
+                            # us this info since the user may have replaced the
+                            # previous mmap'd array
+                            memmap_array = _get_array_memmap(hdu.data)
+
+                            if memmap_array is not None:
+                                memmap_array.flush()
                             else:
                                 hdu._file.seek(hdu._datLoc)
                                 # TODO: Fix this once new HDU writing API is settled on
@@ -743,30 +777,33 @@ class HDUList(list, _Verify):
                     hdu.header._mod = False
                     hdu.header.ascard._mod = False
 
-        if single_thread:
-            if keyboard_interrupt_sent:
-                raise KeyboardInterrupt
-
-            if old_handler is not None:
-                signal.signal(signal.SIGINT, old_handler)
-            else:
-                signal.signal(signal.SIGINT, signal.SIG_DFL)
-
     def update_extend(self):
         """
         Make sure that if the primary header needs the keyword
         ``EXTEND`` that it has it and it is correct.
         """
+
+        if not len(self):
+            return
+
+        if not isinstance(self[0], PrimaryHDU):
+            # A PrimaryHDU will be automatically inserted at some point, but it
+            # might not have been added yet
+            return
+
         hdr = self[0].header
-        if 'extend' in hdr:
-            if (hdr['extend'] == False):
-                hdr['extend'] = True
-        else:
-            if hdr['naxis'] == 0:
-                hdr.update('extend', True, after='naxis')
+
+        if 'EXTEND' in hdr:
+            if len(self) > 1 and hdr['EXTEND'] == False:
+                hdr['EXTEND'] = True
+            elif len(self) == 1 and hdr['EXTEND'] == True:
+                hdr['EXTEND'] = False
+        elif len(self) > 1:
+            if hdr['NAXIS'] == 0:
+                hdr.update('EXTEND', True, after='NAXIS')
             else:
-                n = hdr['naxis']
-                hdr.update('extend', True, after='naxis' + str(n))
+                n = hdr['NAXIS']
+                hdr.update('EXTEND', True, after='NAXIS' + str(n))
 
     @_with_extensions
     def writeto(self, fileobj, output_verify='exception', clobber=False,
@@ -788,12 +825,6 @@ class HDUList(list, _Verify):
         clobber : bool
             When `True`, overwrite the output file if exists.
 
-        classExtensions : dict
-            A dictionary that maps pyfits classes to extensions of
-            those classes.  When present in the dictionary, the
-            extension class will be constructed in place of the pyfits
-            class.
-
         checksum : bool
             When `True` adds both ``DATASUM`` and ``CHECKSUM`` cards
             to the headers of all HDU's written to the file.
@@ -831,8 +862,7 @@ class HDUList(list, _Verify):
                 raise IOError("File '%s' already exists." % filename)
 
         # make sure the EXTEND keyword is there if there is extension
-        if len(self) > 1:
-            self.update_extend()
+        self.update_extend()
 
         mode = 'copyonwrite'
         for key, val in PYTHON_MODES.iteritems():
@@ -994,12 +1024,6 @@ class HDUList(list, _Verify):
 
             # determine if any of the HDU is resized
             for hdu in self:
-                # A shortcut to let an HDU tell us for itself whether or not it
-                # will be resized
-                if hasattr(hdu, '_resize') and hdu._resize:
-                    self._resize = True
-                    break
-
                 # Header:
                 # Add 1 to .ascard to include the END card
                 _nch80 = sum([card._ncards() for card in hdu.header.ascard])
