@@ -34,6 +34,16 @@ class _ImageBaseHDU(_ValidHDU):
                'uint32':32, 'int64':64, 'uint64':64,
                'float32':-32, 'float64':-64}
 
+    standard_keyword_comments = {
+        'SIMPLE': 'conforms to FITS standard',
+        'XTENSION': 'Image extension',
+        'BITPIX': 'array data type',
+        'NAXIS': 'number of array dimensions',
+        'GROUPS': 'has groups',
+        'PCOUNT': 'number of parameters',
+        'GCOUNT': 'number of groups'
+    }
+
     def __init__(self, data=None, header=None, do_not_scale_image_data=False,
                  uint=False, **kwargs):
         from pyfits.hdu.groups import GroupsHDU
@@ -42,16 +52,20 @@ class _ImageBaseHDU(_ValidHDU):
 
         if header is not None:
             if not isinstance(header, Header):
+                # TODO: Instead maybe try initializing a new Header object from
+                # whatever is passed in as the header--there are various types
+                # of objects that could work for this...
                 raise ValueError('header must be a Header object')
 
-
         if data is DELAYED:
-
-            # this should never happen
+            # Presumably if data is DELAYED then this HDU is coming from an
+            # open file, and was not created in memory
             if header is None:
+                # this should never happen
                 raise ValueError('No header to setup HDU.')
 
-            # if the file is read the first time, no need to copy, and keep it unchanged
+            # if the file is read the first time, no need to copy, and keep it
+            # unchanged
             else:
                 self._header = header
         else:
@@ -59,21 +73,25 @@ class _ImageBaseHDU(_ValidHDU):
             # PrimaryHDU and GroupsHDU subclasses
             # construct a list of cards of minimal header
             if isinstance(self, ExtensionHDU):
-                c0 = Card('XTENSION', 'IMAGE', 'Image extension')
+                c0 = Card('XTENSION', 'IMAGE',
+                          self.standard_keyword_comments['XTENSION'])
             else:
-                c0 = Card('SIMPLE', True, 'conforms to FITS standard')
-
+                c0 = Card('SIMPLE', True,
+                          self.standard_keyword_comments['SIMPLE'])
             _list = CardList([
                 c0,
-                Card('BITPIX',    8, 'array data type'),
-                Card('NAXIS',     0, 'number of array dimensions'),
-                ])
+                Card('BITPIX',    8, self.standard_keyword_comments['BITPIX']),
+                Card('NAXIS',     0, self.standard_keyword_comments['NAXIS'])])
+
             if isinstance(self, GroupsHDU):
-                _list.append(Card('GROUPS', True, 'has groups'))
+                _list.append(Card('GROUPS', True,
+                                  self.standard_keyword_comments['GROUPS']))
 
             if isinstance(self, (ExtensionHDU, GroupsHDU)):
-                _list.append(Card('PCOUNT',    0, 'number of parameters'))
-                _list.append(Card('GCOUNT',    1, 'number of groups'))
+                _list.append(Card('PCOUNT',    0,
+                                  self.standard_keyword_comments['PCOUNT']))
+                _list.append(Card('GCOUNT',    1,
+                                  self.standard_keyword_comments['GCOUNT']))
 
             if header is not None:
                 hcopy = header.copy(strip=True)
@@ -83,7 +101,6 @@ class _ImageBaseHDU(_ValidHDU):
 
         self._do_not_scale_image_data = do_not_scale_image_data
         self._uint = uint
-        self._rescaled = False
 
         if do_not_scale_image_data:
             self._bzero = 0
@@ -91,14 +108,23 @@ class _ImageBaseHDU(_ValidHDU):
         else:
             self._bzero = self._header.get('BZERO', 0)
             self._bscale = self._header.get('BSCALE', 1)
-            if not (self._bzero == 0 and self._bscale == 1):
-                self._resize = True
 
-        self._bitpix = self._header['BITPIX']
+        # Save off other important values from the header needed to interpret
+        # the image data
+        self._axes = [self._header.get('NAXIS' + str(axis + 1), 0)
+                      for axis in xrange(self._header.get('NAXIS', 0))]
+        self._bitpix = self._header.get('BITPIX', 8)
+        self._gcount = self._header.get('GCOUNT', 1)
+        self._pcount = self._header.get('PCOUNT', 0)
+        self._blank = self._header.get('BLANK')
 
         if 'name' in kwargs and kwargs['name']:
             self.name = kwargs['name']
 
+        # Set to True if the data or header is replaced, indicating that
+        # update_header should be called
+        self._modified = False
+
         if data is DELAYED:
             return
         else:
@@ -118,20 +144,38 @@ class _ImageBaseHDU(_ValidHDU):
     def section(self):
         return Section(self)
 
-    @lazyproperty
+
+    @property
+    def shape(self):
+        """
+        Shape of the image array--should be equivalent to ``self.data.shape``.
+        """
+
+        # Determine from the values read from the header
+        return tuple(reversed(self._axes))
+
+    def header(self):
+        return self._header
+
+    def _set_header(self, header):
+        self._header = header
+        self._modified = True
+        self.update_header()
+
+    header = property(header, _set_header)
+
     def data(self):
-        if self._header['NAXIS'] < 1:
+        if len(self._axes) < 1:
             return
 
-        dims = self._dimShape()
+        # Numpy array dimensions are in row/column order
         code = _ImageBaseHDU.NumCode[self._bitpix]
 
         raw_data = self._file.readarray(offset=self._datLoc, dtype=code,
-                                        shape=dims)
+                                        shape=self.shape)
         raw_data.dtype = raw_data.dtype.newbyteorder('>')
 
-        if (self._bzero == 0 and self._bscale == 1 and
-            'BLANK' not in self._header):
+        if (self._bzero == 0 and self._bscale == 1 and self._blank is None):
             # No further conversion of the data is necessary
             return raw_data
 
@@ -140,13 +184,11 @@ class _ImageBaseHDU(_ValidHDU):
             data = self._convert_pseudo_unsigned(raw_data)
 
         if data is None:
-            # In these cases, we end up with
-            # floating-point arrays and have to apply
-            # bscale and bzero. We may have to handle
-            # BLANK and convert to NaN in the resulting
-            # floating-point arrays.
-            if 'BLANK' in self._header:
-                blanks = raw_data.flat == self._header['BLANK']
+            # In these cases, we end up with floating-point arrays and have to
+            # apply bscale and bzero. We may have to handle BLANK and convert
+            # to NaN in the resulting floating-point arrays.
+            if self._blank is not None:
+                blanks = raw_data.flat == self._blank
                 # The size of blanks in bytes is the number of elements in
                 # raw_data.flat.  However, if we use np.where instead we will
                 # only use 8 bytes for each index where the condition is true.
@@ -172,52 +214,98 @@ class _ImageBaseHDU(_ValidHDU):
             if self._bzero != 0:
                 data += self._bzero
 
-            if 'BLANK' in self._header:
+            if self._blank is not None:
                 data.flat[blanks] = np.nan
 
         self._update_header_scale_info(data.dtype)
         self._bitpix = self._header['BITPIX']
+        self._bzero = 0
+        self._bscale = 1
 
         return data
 
+    def _set_data(self, data):
+        self.__dict__['data'] = data
+        self._modified = True
+        if self.data is not None and not isinstance(data, np.ndarray):
+            # Try to coerce the data into a numpy array--this will work, on
+            # some level, for most objects
+            try:
+                data = np.array(data)
+            except:
+                raise TypeError('data object %r could not be coerced into an '
+                                'ndarray' % data)
+
+        if isinstance(data, np.ndarray):
+            self._bitpix = _ImageBaseHDU.ImgCode[data.dtype.name]
+            self._axes = list(data.shape)
+            self._axes.reverse()
+        elif self.data is None:
+            self._axes = []
+        else:
+            raise ValueError('not a valid data array')
+
+        self.update_header()
+
+    data = lazyproperty(data, _set_data)
+
     def update_header(self):
         """
         Update the header keywords to agree with the data.
         """
 
+        if (not self._modified and not self._header._mod and
+            not (self._data_loaded and self.shape == self.data.shape)):
+            # Not likely that anything needs updating
+            return
+
         old_naxis = self._header.get('NAXIS', 0)
 
-        if isinstance(self.data, np.ndarray):
-            self._bitpix = _ImageBaseHDU.ImgCode[self.data.dtype.name]
-            self._header['BITPIX'] = self._bitpix
-            axes = list(self.data.shape)
-            axes.reverse()
-
-        elif self.data is None:
-            axes = []
+        if 'BITPIX' not in self._header:
+            bitpix_comment = self.standard_keyword_comments['BITPIX']
         else:
-            raise ValueError('incorrect array type')
+            bitpix_comment = self._header.ascard['BITPIX'].comment
 
-        self._header['NAXIS'] = len(axes)
+        # Update the BITPIX keyword and ensure it's in the correct
+        # location in the header
+        self._header.update('BITPIX', self._bitpix, bitpix_comment, after=0)
+
+        # If the data's shape has changed (this may have happened without our
+        # noticing either via a direct update to the data.shape attribute) we
+        # need to update the internal self._axes
+        if self._data_loaded and self.shape != self.data.shape:
+            self._axes = list(self.data.shape)
+            self._axes.reverse()
+
+        # Update the NAXIS keyword and ensure it's in the correct location in
+        # the header
+        if 'NAXIS' in self._header:
+            naxis_comment = self._header.ascard['NAXIS'].comment
+        else:
+            naxis_comment = self.standard_keyword_comments['NAXIS']
+        self._header.update('NAXIS', len(self._axes), naxis_comment,
+                            after='BITPIX')
 
         # add NAXISi if it does not exist
-        for idx, axis in enumerate(axes):
+        for idx, axis in enumerate(self._axes):
             try:
                 self._header['NAXIS'+ str(idx + 1)] = axis
             except KeyError:
                 if (idx == 0):
-                    after = 'naxis'
+                    after = 'NAXIS'
                 else :
-                    after = 'naxis' + str(idx)
+                    after = 'NAXIS' + str(idx)
                 self._header.update('naxis' + str(idx + 1), axis, after=after)
 
         # delete extra NAXISi's
-        for idx in range(len(axes)+1, old_naxis+1):
+        for idx in range(len(self._axes) + 1, old_naxis + 1):
             try:
                 del self._header.ascard['NAXIS' + str(idx)]
             except KeyError:
                 pass
 
+        self._modified = False
+
     def _update_header_scale_info(self, dtype=None):
         if (not self._do_not_scale_image_data and
             not (self._bzero == 0 and self._bscale == 1)):
@@ -229,7 +317,10 @@ class _ImageBaseHDU(_ValidHDU):
             if dtype is not None:
                 self._header['BITPIX'] = _ImageBaseHDU.ImgCode[dtype.name]
 
-            self._rescaled = True
+            # These are some internal tweaks that update_header doesn't need to
+            # know about, and it won't know if we tell it the header is
+            # unmodified
+            self._header._mod = 0
 
     def scale(self, type=None, option="old", bscale=1, bzero=0):
         """
@@ -316,18 +407,14 @@ class _ImageBaseHDU(_ValidHDU):
         self._header['BITPIX'] = self._bitpix
 
     def _verify(self, option='warn'):
-        if not self._data_loaded:
-            # This can affect the presence and order of certain header
-            # keywords, so run it before running the verification
-            self._update_header_scale_info()
+        # update_header can fix some things that would otherwise cause
+        # verification to fail, so do that now...
+        self.update_header()
 
         return super(_ImageBaseHDU, self)._verify(option)
 
     def _writeheader(self, fileobj, checksum=False):
-        if self._data_loaded:
-            self.update_header()
-        else:
-            self._update_header_scale_info()
+        self.update_header()
         return super(_ImageBaseHDU, self)._writeheader(fileobj, checksum)
 
     def _writedata_internal(self, fileobj):
@@ -364,15 +451,16 @@ class _ImageBaseHDU(_ValidHDU):
 
             size += output.size * output.itemsize
 
-        if self._rescaled:
-            # If the data was rescaled then the file written to was
-            # automatically resized on writing.  But now that the scaled data
-            # has been written we don't necessarily need to resize on
-            # subsequent writes
-            self._resize = False
-
         return size
 
+    def _writeto(self, fileobj, checksum=False):
+        if not self._data_loaded:
+            # Normally this is done when the data is loaded, but since the data
+            # is not loaded yet we need to update the header appropriately
+            # before writing it
+            self._update_header_scale_info()
+        return super(_ImageBaseHDU, self)._writeto(fileobj, checksum=checksum)
+
     def _dtype_for_bitpix(self):
         """
         Determine the dtype that the data should be converted to depending on
@@ -418,62 +506,29 @@ class _ImageBaseHDU(_ValidHDU):
             data -= np.uint64(1 << (bits - 1))
             return data
 
-    def _dimShape(self):
-        """
-        Returns a tuple of image dimensions, reverse the order of ``NAXIS``.
-        """
-        naxis = self._header['NAXIS']
-        axes = naxis*[0]
-        for idx in range(naxis):
-            axes[idx] = self._header['NAXIS' + str(idx + 1)]
-        axes.reverse()
-#        print "axes in _dimShape line 2081:",axes
-        return tuple(axes)
-
     # TODO: Move the GroupsHDU-specific summary code to GroupsHDU itself
     def _summary(self):
         """
         Summarize the HDU: name, dimensions, and formats.
         """
-        from pyfits.hdu.groups import GroupsHDU
 
         class_name  = self.__class__.__name__
 
         # if data is touched, use data info.
         if self._data_loaded:
             if self.data is None:
-                _shape, _format = (), ''
+                format = ''
             else:
-
-                # the shape will be in the order of NAXIS's which is the
-                # reverse of the numarray shape
-                if isinstance(self, GroupsHDU):
-                    _shape = list(self.data.data.shape)[1:]
-                    _format = \
-                       self.data.dtype.fields[self.data.dtype.names[0]][0].name
-                else:
-                    _shape = list(self.data.shape)
-                    _format = self.data.dtype.name
-                _shape.reverse()
-                _shape = tuple(_shape)
-                _format = _format[_format.rfind('.')+1:]
-
-        # if data is not touched yet, use header info.
+                format = self.data.dtype.name
+                format = format[format.rfind('.')+1:]
         else:
-            _shape = ()
-            for idx in range(self._header['NAXIS']):
-                if isinstance(self, GroupsHDU) and idx == 0:
-                    continue
-                _shape += (self._header['NAXIS' + str(idx + 1)],)
-            _format = self.NumCode[self._bitpix]
+            # if data is not touched yet, use header info.
+            format = self.NumCode[self._bitpix]
 
-        if isinstance(self, GroupsHDU):
-            _gcount = '   %d Groups  %d Parameters' \
-                      % (self._header['GCOUNT'], self._header['PCOUNT'])
-        else:
-            _gcount = ''
-        return (self.name, class_name, len(self._header.ascard), _shape,
-                _format, _gcount)
+        # Display shape in FITS-order
+        shape = tuple(reversed(self.shape))
+
+        return (self.name, class_name, len(self._header), shape, format, '')
 
     def _calculate_datasum(self, blocking):
         """
@@ -532,11 +587,11 @@ class Section(object):
         dims = []
         if not isinstance(key, tuple):
             key = (key,)
-        naxis = self.hdu.header['NAXIS']
+        naxis = len(self.hdu.shape)
         if naxis < len(key):
             raise IndexError('too many indices')
         elif naxis > len(key):
-            key = key + (slice(None),) * (naxis-len(key))
+            key = key + (slice(None),) * (naxis - len(key))
 
         offset = 0
 
@@ -544,9 +599,9 @@ class Section(object):
         # scope leak defect
         idx = 0
         for idx in range(naxis):
-            _naxis = self.hdu.header['NAXIS'+ str(naxis - idx)]
-            indx = _iswholeline(key[idx], _naxis)
-            offset = offset * _naxis + indx.offset
+            axis = self.hdu.shape[idx]
+            indx = _iswholeline(key[idx], axis)
+            offset = offset * axis + indx.offset
 
             # all elements after the first WholeLine must be WholeLine or
             # OnePointAxis
@@ -559,23 +614,30 @@ class Section(object):
         contiguousSubsection = True
 
         for jdx in range(idx + 1, naxis):
-            _naxis = self.hdu.header['NAXIS' + str(naxis - jdx)]
-            indx = _iswholeline(key[jdx], _naxis)
+            axis = self.hdu.shape[jdx]
+            indx = _iswholeline(key[jdx], axis)
             dims.append(indx.npts)
             if not isinstance(indx, _WholeLine):
                 contiguousSubsection = False
 
             # the offset needs to multiply the length of all remaining axes
             else:
-                offset *= _naxis
+                offset *= axis
 
         if contiguousSubsection:
             if not dims:
                 dims = [1]
 
-            _bitpix = self.hdu._bitpix
-            code = _ImageBaseHDU.NumCode[_bitpix]
-            offset = self.hdu._datLoc + (offset * abs(_bitpix) // 8)
+            dims = tuple(dims)
+
+            bitpix = self.hdu._bitpix
+            offset = self.hdu._datLoc + (offset * abs(bitpix) // 8)
+            # If the data is already loaded use whatever dtype it was scaled
+            # to, else figure it out from the bitpix...
+            if self.hdu._data_loaded:
+                code = self.hdu.data.dtype.name
+            else:
+                code = _ImageBaseHDU.NumCode[bitpix]
             raw_data = self.hdu._file.readarray(offset=offset, dtype=code,
                                                 shape=dims)
             raw_data.dtype = raw_data.dtype.newbyteorder('>')
@@ -583,15 +645,16 @@ class Section(object):
         else:
             data = self._getdata(key)
 
-        converted_data = self.hdu._convert_pseudo_unsigned(data)
-        if converted_data is not None:
-            data = converted_data
+        if not (self.hdu._bzero == 0 and self.hdu._bscale == 1):
+            converted_data = self.hdu._convert_pseudo_unsigned(data)
+            if converted_data is not None:
+                data = converted_data
 
         return data
 
     def _getdata(self, keys):
         out = []
-        naxis = self.hdu.header['NAXIS']
+        naxis = len(self.hdu.shape)
 
         # Determine the number of slices in the set of input keys.
         # If there is only one slice then the result is a one dimensional
@@ -605,8 +668,8 @@ class Section(object):
             if isinstance(key, slice):
                 # OK, this element is a slice so see if we can get the data for
                 # each element of the slice.
-                _naxis = self.hdu.header['NAXIS' + str(naxis - idx)]
-                ns = _normalize_slice(key, _naxis)
+                axis = self.hdu.shape[idx]
+                ns = _normalize_slice(key, axis)
 
                 for k in range(ns.start, ns.stop):
                     key1 = list(keys)
@@ -680,10 +743,10 @@ class PrimaryHDU(_ImageBaseHDU):
 
         # insert the keywords EXTEND
         if header is None:
-            dim = repr(self._header['NAXIS'])
-            if dim == '0':
+            dim = self._header['NAXIS']
+            if dim == 0:
                 dim = ''
-            self._header.update('EXTEND', True, after='NAXIS' + dim)
+            self._header.update('EXTEND', True, after='NAXIS' + str(dim))
 
     @classmethod
     def match_header(cls, header):
@@ -692,6 +755,17 @@ class PrimaryHDU(_ImageBaseHDU):
                ('GROUPS' not in header or header['GROUPS'] != True) and \
                card.value == True
 
+    def update_header(self):
+        super(PrimaryHDU, self).update_header()
+
+        # Update the position of the EXTEND keyword if it already exists
+        if 'EXTEND' in self._header:
+            if len(self._axes):
+                after = 'NAXIS' + str(len(self._axes))
+            else:
+                after = 'NAXIS'
+            self._header.update('EXTEND', self._header['EXTEND'], after=after)
+
     def _verify(self, option='warn'):
         errs = super(PrimaryHDU, self)._verify(option=option)
 
