@@ -7,7 +7,7 @@ import numpy as np
 
 import pyfits
 from pyfits.tests import PyfitsTestCase
-from pyfits.tests.util import catch_warnings, BytesIO
+from pyfits.tests.util import catch_warnings, BytesIO, ignore_warnings
 
 from nose.tools import assert_equal, assert_raises, assert_true
 
@@ -36,7 +36,7 @@ class TestHDUListFunctions(PyfitsTestCase):
             assert_equal(res['filename'], self.data('checksum.fits'))
             assert_equal(res['datLoc'], kwargs.get('datLoc', 8640))
             assert_equal(res['hdrLoc'], kwargs.get('hdrLoc', 0))
-            assert_equal(res['filemode'], 'copyonwrite')
+            assert_equal(res['filemode'], 'readonly')
 
         res = hdul.fileinfo(1)
         test_fileinfo(datLoc=17280, hdrLoc=11520)
@@ -345,7 +345,7 @@ class TestHDUListFunctions(PyfitsTestCase):
         hdu = pyfits.PrimaryHDU(np.arange(100, dtype=np.int32))
         hdul = pyfits.HDUList()
         hdul.append(hdu)
-        tmpfile = open(self.temp('tmpfile.fits'), 'w')
+        tmpfile = open(self.temp('tmpfile.fits'), 'wb')
         hdul.writeto(tmpfile)
         tmpfile.close()
 
@@ -356,7 +356,7 @@ class TestHDUListFunctions(PyfitsTestCase):
 
     def test_file_like_2(self):
         hdu = pyfits.PrimaryHDU(np.arange(100, dtype=np.int32))
-        tmpfile = open(self.temp('tmpfile.fits'), 'w')
+        tmpfile = open(self.temp('tmpfile.fits'), 'wb')
         hdul = pyfits.open(tmpfile, mode='ostream')
         hdul.append(hdu)
         hdul.flush()
@@ -369,7 +369,7 @@ class TestHDUListFunctions(PyfitsTestCase):
 
     def test_file_like_3(self):
 
-        tmpfile = open(self.temp('tmpfile.fits'), 'w')
+        tmpfile = open(self.temp('tmpfile.fits'), 'wb')
         pyfits.writeto(tmpfile, np.arange(100, dtype=np.int32))
         tmpfile.close()
         info = [(0, 'PRIMARY', 'PrimaryHDU', 5, (100,), 'int32', '')]
@@ -434,3 +434,44 @@ class TestHDUListFunctions(PyfitsTestCase):
 
         assert_true('EXTEND' in hdul[0].header)
         assert_equal(hdul[0].header['EXTEND'], True)
+
+    def test_new_hdulist_extend_keyword(self):
+        """
+        Tests that adding a PrimaryHDU to a new HDUList object updates the
+        EXTEND keyword on that HDU.  Regression test for #114.
+        """
+
+        h0 = pyfits.Header()
+        hdu = pyfits.PrimaryHDU(header=h0)
+        sci = pyfits.ImageHDU(data=np.array(10))
+        image = pyfits.HDUList([hdu, sci])
+        image.writeto(self.temp('temp.fits'))
+        assert_true('EXTEND' in hdu.header)
+        assert_equal(hdu.header['EXTEND'], True)
+
+    def test_replace_memmaped_array(self):
+        # Copy the original before we modify it
+        hdul = pyfits.open(self.data('test0.fits'))
+        hdul.writeto(self.temp('temp.fits'))
+
+        hdul = pyfits.open(self.temp('temp.fits'), mode='update', memmap=True)
+        old_data = hdul[1].data.copy()
+        hdul[1].data = hdul[1].data + 1
+        hdul.close()
+        hdul = pyfits.open(self.temp('temp.fits'), memmap=True)
+        assert_true(((old_data + 1) == hdul[1].data).all())
+
+    def test_open_file_with_end_padding(self):
+        """Regression test for #106; open files with end padding bytes."""
+
+        hdul = pyfits.open(self.data('test0.fits'),
+                           do_not_scale_image_data=True)
+        info = hdul.info(output=False)
+        hdul.writeto(self.temp('temp.fits'))
+        with open(self.temp('temp.fits'),'ab') as f:
+            f.seek(0, os.SEEK_END)
+            f.write('\0'.encode('latin1') * 2880)
+        with ignore_warnings():
+            assert_equal(info,
+                         pyfits.info(self.temp('temp.fits'), output=False,
+                                     do_not_scale_image_data=True))
