@@ -7,6 +7,7 @@ from numpy import char as chararray
 import pyfits
 from pyfits.util import decode_ascii
 from pyfits.tests import PyfitsTestCase
+from pyfits.tests.util import ignore_warnings
 
 from nose.tools import (assert_equal, assert_not_equal, assert_raises,
                         assert_true)
@@ -144,7 +145,7 @@ class TestTableFunctions(PyfitsTestCase):
         assert_equal(tbhdu.data[1][4], a5[1])
         assert_true((tbhdu.data[1][5] == a6[1].view('bool')).all())
         assert_equal(tbhdu.data[1][6], a7[1])
-        assert_equal(tbhdu.data[1][7].all(), a8[1].all())
+        assert_true((tbhdu.data[1][7] == a8[1]).all())
 
         # and a column like this:
         assert_equal(str(tbhdu.data.field('abc')), "['abc' 'def' 'xx']")
@@ -201,7 +202,7 @@ class TestTableFunctions(PyfitsTestCase):
         assert_equal(str(np.rec.recarray.field(t[1].data,'c4')), "[84 84]")
 
         # look at data column-wise
-        assert_equal(t[1].data.field(0).all(), np.array([1, 2]).all())
+        assert_true((t[1].data.field(0) == np.array([1, 2])).all())
 
         # When there are scaled columns, the raw data are in data._parent
 
@@ -220,11 +221,11 @@ class TestTableFunctions(PyfitsTestCase):
 
         # Test slicing
         a2 = a[1].data[2:][2:]
-        ra2 = np.rec.array([(345.0,345)],names='c1, c2')
+        ra2 = np.rec.array([(345.0,345)], names='c1, c2')
 
         assert_equal(comparerecords(a2, ra2), True)
 
-        assert_equal(a2.field(1).all(),np.array([345]).all())
+        assert_true((a2.field(1) == np.array([345])).all())
 
         ra3 = np.rec.array([
             (10.123000144958496, 37),
@@ -263,8 +264,7 @@ class TestTableFunctions(PyfitsTestCase):
         hdu_list.writeto(self.temp('toto.fits'), clobber=True)
         toto = pyfits.open(self.temp('toto.fits'))
         q = toto[1].data.field('QUAL_SPE')
-        assert_equal(q[0][4:8].all(),
-                         np.array([0, 0, 0, 0],dtype=np.uint8).all())
+        assert_true((q[0][4:8] == np.array([0, 0, 0, 0],dtype=np.uint8)).all())
         toto.close()
 
     def test_extend_variable_length_array(self):
@@ -298,7 +298,7 @@ class TestTableFunctions(PyfitsTestCase):
         rfiHDU = hduL['RFI']
         data = rfiHDU.data
         channelsOut = data.field('Channels')[0]
-        assert_equal(channelsIn.all(),channelsOut.all())
+        assert_true((channelsIn == channelsOut).all())
         hduL.close()
 
     def test_column_endianness(self):
@@ -356,20 +356,20 @@ class TestTableFunctions(PyfitsTestCase):
 
     def test_new_table_from_recarray(self):
         bright = np.rec.array([(1,'Serius',-1.45,'A1V'),
-                            (2,'Canopys',-0.73,'F0Ib'),
-                            (3,'Rigil Kent',-0.1,'G2V')],
-                           formats='int16,a20,float32,a10',
-                           names='order,name,mag,Sp')
+                               (2,'Canopys',-0.73,'F0Ib'),
+                               (3,'Rigil Kent',-0.1,'G2V')],
+                              formats='int16,a20,float32,a10',
+                              names='order,name,mag,Sp')
         hdu=pyfits.new_table(bright,nrows=2,tbtype='TableHDU')
 
         # Verify that all ndarray objects within the HDU reference the
         # same ndarray.
         assert_equal(id(hdu.data._coldefs.columns[0].array),
-                         id(hdu.data._coldefs._arrays[0]))
+                     id(hdu.data._coldefs._arrays[0]))
         assert_equal(id(hdu.data._coldefs.columns[0].array),
-                         id(hdu.columns.data[0].array))
+                     id(hdu.columns.columns[0].array))
         assert_equal(id(hdu.data._coldefs.columns[0].array),
-                         id(hdu.columns._arrays[0]))
+                     id(hdu.columns._arrays[0]))
 
         # Ensure I can change the value of one data element and it effects
         # all of the others.
@@ -410,37 +410,38 @@ class TestTableFunctions(PyfitsTestCase):
         assert_equal(hdu.columns._arrays[0][0], 800)
         assert_equal(hdu.columns.data[0].array[0], 800)
 
-        assert_equal(hdu.data.field(0).all(),
-                         np.array([1, 2],dtype=np.int16).all())
+        assert_true((hdu.data.field(0) ==
+                     np.array([800, 2],dtype=np.int16)).all())
         assert_equal(hdu.data[0][1], 'Serius')
         assert_equal(hdu.data[1][1], 'Canopys')
-        assert_equal(hdu.data.field(2).all(),
-                         np.array([-1.45, -0.73], dtype=np.float32).all())
+        assert_true((hdu.data.field(2) ==
+                     np.array([-1.45, -0.73], dtype=np.float32)).all())
         assert_equal(hdu.data[0][3], 'A1V')
         assert_equal(hdu.data[1][3], 'F0Ib')
-        hdu.writeto(self.temp('toto.fits'), clobber=True)
-        hdul = pyfits.open(self.temp('toto.fits'))
-        assert_equal(hdul[1].data.field(0).all(),
-                         np.array([1, 2], dtype=np.int16).all())
-        assert_equal(hdul[1].data[0][1], 'Serius')
-        assert_equal(hdul[1].data[1][1], 'Canopys')
-        assert_equal(hdul[1].data.field(2).all(),
-                         np.array([-1.45, -0.73], dtype=np.float32).all())
-        assert_equal(hdul[1].data[0][3], 'A1V')
-        assert_equal(hdul[1].data[1][3], 'F0Ib')
 
-        hdul.close()
+        with ignore_warnings():
+            hdu.writeto(self.temp('toto.fits'), clobber=True)
 
-        hdu=pyfits.new_table(bright,nrows=2)
-        tmp=np.rec.array([(1,'Serius',-1.45,'A1V'),
-                          (2,'Canopys',-0.73,'F0Ib')],
-                         formats='int16,a20,float32,a10',
-                         names='order,name,mag,Sp')
-        assert_equal(comparerecords(hdu.data,tmp), True)
+        with pyfits.open(self.temp('toto.fits')) as hdul:
+            assert_true((hdul[1].data.field(0) ==
+                         np.array([800, 2], dtype=np.int16)).all())
+            assert_equal(hdul[1].data[0][1], 'Serius')
+            assert_equal(hdul[1].data[1][1], 'Canopys')
+            assert_true((hdul[1].data.field(2) ==
+                         np.array([-1.45, -0.73], dtype=np.float32)).all())
+            assert_equal(hdul[1].data[0][3], 'A1V')
+            assert_equal(hdul[1].data[1][3], 'F0Ib')
+        del hdul
+
+        hdu = pyfits.new_table(bright,nrows=2)
+        tmp = np.rec.array([(1,'Serius',-1.45,'A1V'),
+                            (2,'Canopys',-0.73,'F0Ib')],
+                            formats='int16,a20,float32,a10',
+                            names='order,name,mag,Sp')
+        assert_true(comparerecords(hdu.data, tmp))
         hdu.writeto(self.temp('toto.fits'), clobber=True)
-        hdul = pyfits.open(self.temp('toto.fits'))
-        assert_equal(comparerecords(hdu.data,hdul[1].data),True)
-        hdul.close()
+        with pyfits.open(self.temp('toto.fits')) as hdul:
+            assert_true(comparerecords(hdu.data, hdul[1].data))
 
     def test_new_fitsrec(self):
         """
@@ -810,8 +811,8 @@ class TestTableFunctions(PyfitsTestCase):
         a,b,c = row[1:4]
         assert_equal(a, counts[2])
         assert_equal(b, '0.0')
-        assert_equal(c.all(), np.array([ 0.,  0.,  0.,  0.,  0.],
-                                           dtype=np.float32).all())
+        assert_true((c == np.array([ 0.,  0.,  0.,  0.,  0.],
+                                   dtype=np.float32)).all())
         row['counts'] = 310
         assert_equal(row['counts'], 310)
 
@@ -930,22 +931,22 @@ class TestTableFunctions(PyfitsTestCase):
         assert_equal(tbhdu.data._coldefs._arrays[1][0], 312)
         assert_equal(tbhdu.data._coldefs.columns[1].array[0], 312)
         assert_equal(tbhdu.columns._arrays[1][0], 312)
-        assert_equal(tbhdu.columns.data[1].array[0], 312)
-        assert_equal(tbhdu.columns.data[0].array[0], 'NGC1')
-        assert_equal(tbhdu.columns.data[2].array[0], '0.0')
-        assert_equal(tbhdu.columns.data[3].array[0].all(),
-                         np.array([0., 0., 0., 0., 0.],dtype=np.float32).all())
+        assert_equal(tbhdu.columns.columns[1].array[0], 312)
+        assert_equal(tbhdu.columns.columns[0].array[0], 'NGC1')
+        assert_equal(tbhdu.columns.columns[2].array[0], '0.0')
+        assert_true((tbhdu.columns.columns[3].array[0] ==
+                     np.array([0., 0., 0., 0., 0.],dtype=np.float32)).all())
         assert_equal(tbhdu.columns.data[4].array[0], True)
 
         assert_equal(tbhdu.data[3][1], 33)
         assert_equal(tbhdu.data._coldefs._arrays[1][3], 33)
         assert_equal(tbhdu.data._coldefs.columns[1].array[3], 33)
         assert_equal(tbhdu.columns._arrays[1][3], 33)
-        assert_equal(tbhdu.columns.data[1].array[3], 33)
-        assert_equal(tbhdu.columns.data[0].array[3], 'JIM1')
-        assert_equal(tbhdu.columns.data[2].array[3], 'A Note')
-        assert_equal(tbhdu.columns.data[3].array[3].all(),
-                         np.array([1., 2., 3., 4., 5.],dtype=np.float32).all())
+        assert_equal(tbhdu.columns.columns[1].array[3], 33)
+        assert_equal(tbhdu.columns.columns[0].array[3], 'JIM1')
+        assert_equal(tbhdu.columns.columns[2].array[3], 'A Note')
+        assert_true((tbhdu.columns.columns[3].array[3] ==
+                     np.array([1., 2., 3., 4., 5.],dtype=np.float32)).all())
         assert_equal(tbhdu.columns.data[4].array[3], True)
 
     def test_assign_multiple_rows_to_table(self):
@@ -992,29 +993,29 @@ class TestTableFunctions(PyfitsTestCase):
         assert_equal(tbhdu2.data._coldefs._arrays[1][0], 312)
         assert_equal(tbhdu2.data._coldefs.columns[1].array[0], 312)
         assert_equal(tbhdu2.columns._arrays[1][0], 312)
-        assert_equal(tbhdu2.columns.data[1].array[0], 312)
-        assert_equal(tbhdu2.columns.data[0].array[0], 'NGC1')
-        assert_equal(tbhdu2.columns.data[2].array[0], '0.0')
-        assert_equal(tbhdu2.columns.data[3].array[0].all(),
-                         np.array([0., 0., 0., 0., 0.],dtype=np.float32).all())
+        assert_equal(tbhdu2.columns.columns[1].array[0], 312)
+        assert_equal(tbhdu2.columns.columns[0].array[0], 'NGC1')
+        assert_equal(tbhdu2.columns.columns[2].array[0], '0.0')
+        assert_true((tbhdu2.columns.columns[3].array[0] ==
+                     np.array([0., 0., 0., 0., 0.],dtype=np.float32)).all())
         assert_equal(tbhdu2.columns.data[4].array[0], True)
 
         assert_equal(tbhdu2.data[4][1], 112)
         assert_equal(tbhdu2.data._coldefs._arrays[1][4], 112)
         assert_equal(tbhdu2.data._coldefs.columns[1].array[4], 112)
         assert_equal(tbhdu2.columns._arrays[1][4], 112)
-        assert_equal(tbhdu2.columns.data[1].array[4], 112)
-        assert_equal(tbhdu2.columns.data[0].array[4], 'NGC5')
-        assert_equal(tbhdu2.columns.data[2].array[4], '0.0')
-        assert_equal(tbhdu2.columns.data[3].array[4].all(),
-                         np.array([1., 2., 3., 4., 5.],dtype=np.float32).all())
+        assert_equal(tbhdu2.columns.columns[1].array[4], 112)
+        assert_equal(tbhdu2.columns.columns[0].array[4], 'NGC5')
+        assert_equal(tbhdu2.columns.columns[2].array[4], '0.0')
+        assert_true((tbhdu2.columns.columns[3].array[4] ==
+                     np.array([1., 2., 3., 4., 5.],dtype=np.float32)).all())
         assert_equal(tbhdu2.columns.data[4].array[4], False)
 
-        assert_equal(tbhdu2.columns.data[1].array[8], 0)
-        assert_equal(tbhdu2.columns.data[0].array[8], '0.0')
-        assert_equal(tbhdu2.columns.data[2].array[8], '0.0')
-        assert_equal(tbhdu2.columns.data[3].array[8].all(),
-                         np.array([0., 0., 0., 0., 0.],dtype=np.float32).all())
+        assert_equal(tbhdu2.columns.columns[1].array[8], 0)
+        assert_equal(tbhdu2.columns.columns[0].array[8], '0.0')
+        assert_equal(tbhdu2.columns.columns[2].array[8], '0.0')
+        assert_true((tbhdu2.columns.columns[3].array[8] ==
+                     np.array([0., 0., 0., 0., 0.],dtype=np.float32)).all())
         assert_equal(tbhdu2.columns.data[4].array[8], False)
 
     def test_verify_data_references(self):
@@ -1288,7 +1289,7 @@ class TestTableFunctions(PyfitsTestCase):
         for row in tbhdu1.data:
             for j in range(0,len(row)):
                 if isinstance(row[j], np.ndarray):
-                    assert_equal(row[j].all(), tbhdu.data[i][j].all())
+                    assert_true((row[j] == tbhdu.data[i][j]).all())
                 else:
                     assert_equal(row[j], tbhdu.data[i][j])
             i = i + 1
@@ -1474,21 +1475,17 @@ class TestTableFunctions(PyfitsTestCase):
 
         tbhdu1 = pyfits.new_table(coldefs)
 
-        assert_equal(tbhdu1.data.field('flag')[0].all(),
-                         np.array([True, False],
-                                  dtype = np.bool).all())
-        assert_equal(tbhdu1.data.field('flag')[1].all(),
-                         np.array([False, True],
-                                  dtype = np.bool).all())
+        assert_true((tbhdu1.data.field('flag')[0] ==
+                     np.array([True, False], dtype = np.bool)).all())
+        assert_true((tbhdu1.data.field('flag')[1] ==
+                     np.array([False, True], dtype = np.bool)).all())
 
         tbhdu = pyfits.new_table(tbhdu1.data)
 
-        assert_equal(tbhdu.data.field('flag')[0].all(),
-                         np.array([True, False],
-                                  dtype = np.bool).all())
-        assert_equal(tbhdu.data.field('flag')[1].all(),
-                         np.array([False, True],
-                                  dtype = np.bool).all())
+        assert_true((tbhdu.data.field('flag')[0] ==
+                     np.array([True, False], dtype = np.bool)).all())
+        assert_true((tbhdu.data.field('flag')[1] ==
+                      np.array([False, True], dtype = np.bool)).all())
 
     def test_variable_length_table_format_pd_from_object_array(self):
         a = np.array([np.array([7.2e-20, 7.3e-20]), np.array([0.0]),
@@ -1547,8 +1544,8 @@ class TestTableFunctions(PyfitsTestCase):
     def test_fits_rec_column_access(self):
         t=pyfits.open(self.data('table.fits'))
         tbdata = t[1].data
-        assert_equal(tbdata.V_mag.all(), tbdata.field('V_mag').all())
-        assert_equal(tbdata.V_mag.all(), tbdata['V_mag'].all())
+        assert_true((tbdata.V_mag == tbdata.field('V_mag')).all())
+        assert_true((tbdata.V_mag == tbdata['V_mag']).all())
 
         t.close()
 
@@ -1593,10 +1590,30 @@ class TestTableFunctions(PyfitsTestCase):
                              array=chararray.array(a))
         ahdu = pyfits.new_table([acol])
         assert_equal(ahdu.data.tostring().decode('raw-unicode-escape'), s)
+        ahdu.writeto(self.temp('newtable.fits'))
+        with pyfits.open(self.temp('newtable.fits')) as hdul:
+            assert_equal(hdul[1].data.tostring().decode('raw-unicode-escape'),
+                         s)
+            assert_true((hdul[1].data['MEMNAME'] == a).all())
+        del hdul
 
         ahdu = pyfits.new_table([acol], tbtype='TableHDU')
-        assert_equal(ahdu.data.tostring().decode('raw-unicode-escape'),
+        ahdu.writeto(self.temp('newtable.fits'), clobber=True)
+
+        with pyfits.open(self.temp('newtable.fits')) as hdul:
+            assert_equal(hdul[1].data.tostring().decode('raw-unicode-escape'),
                          s.replace('\x00', ' '))
+            assert_true((hdul[1].data['MEMNAME'] == a).all())
+            ahdu = pyfits.new_table(hdul[1].data.copy())
+        del hdul
+
+        # Now serialize once more as a binary table; padding bytes should
+        # revert to zeroes
+        ahdu.writeto(self.temp('newtable.fits'), clobber=True)
+        with pyfits.open(self.temp('newtable.fits')) as hdul:
+            assert_equal(hdul[1].data.tostring().decode('raw-unicode-escape'),
+                         s)
+            assert_true((hdul[1].data['MEMNAME'] == a).all())
 
     def test_multi_dimensional_columns(self):
         """
@@ -1616,22 +1633,24 @@ class TestTableFunctions(PyfitsTestCase):
 
         thdu.writeto(self.temp('newtable.fits'))
 
-        hdul = pyfits.open(self.temp('newtable.fits'))
-        thdu = hdul[1]
+        with pyfits.open(self.temp('newtable.fits')) as hdul:
+            thdu = hdul[1]
 
-        c1 = thdu.data.field(0)
-        c2 = thdu.data.field(1)
+            c1 = thdu.data.field(0)
+            c2 = thdu.data.field(1)
 
-        hdul.close()
-
-        assert_equal(c1.shape, (3, 3, 2))
-        assert_equal(c2.shape, (3, 2))
-        assert_true((c1 == np.array([[[0, 1], [2, 3], [4, 5]],
-                                     [[6, 7], [8, 9], [0, 1]],
-                                     [[2, 3], [4, 5], [6, 7]]])).all())
-        assert_true((c2 == np.array([['row1', 'row1'],
-                                     ['row2', 'row2'],
-                                     ['row3', 'row3']])).all())
+            assert_equal(c1.shape, (3, 3, 2))
+            assert_equal(c2.shape, (3, 2))
+            assert_true((c1 == np.array([[[0, 1], [2, 3], [4, 5]],
+                                         [[6, 7], [8, 9], [0, 1]],
+                                         [[2, 3], [4, 5], [6, 7]]])).all())
+            assert_true((c2 == np.array([['row1', 'row1'],
+                                         ['row2', 'row2'],
+                                         ['row3', 'row3']])).all())
+        del c1
+        del c2
+        del thdu
+        del hdul
 
         # Test setting the TDIMn header based on the column data
         data = np.zeros(3, dtype=[('x', 'f4'), ('s', 'S5', 4)])
@@ -1649,7 +1668,11 @@ class TestTableFunctions(PyfitsTestCase):
         data = np.zeros(3, dtype=[('x', 'f4'), ('s', 'S5', (4, 3))])
         data['x'] = 1, 2, 3
         data['s'] = 'ok'
-        pyfits.writeto(self.temp('newtable.fits'), data, clobber=True)
+
+        del t
+
+        with ignore_warnings():
+            pyfits.writeto(self.temp('newtable.fits'), data, clobber=True)
 
         t = pyfits.getdata(self.temp('newtable.fits'))
 
@@ -1731,3 +1754,21 @@ class TestTableFunctions(PyfitsTestCase):
         assert_true((t.data['names'] == [1]).all())
         assert_true((t.data['formats'] == [2]).all())
         assert_true((t.data.other == [3]).all())
+
+    def test_table_from_bool_fields(self):
+        """
+        Regression test for #113.
+
+        Tests creating a table from a recarray containing numpy.bool columns.
+        """
+
+        array = np.rec.array([(True, False), (False, True)], formats='|b1,|b1')
+        thdu = pyfits.new_table(array)
+        assert_equal(thdu.columns.formats, ['L', 'L'])
+        assert_true(comparerecords(thdu.data, array))
+
+        # Test round trip
+        thdu.writeto(self.temp('table.fits'))
+        data = pyfits.getdata(self.temp('table.fits'), ext=1)
+        assert_equal(thdu.columns.formats, ['L', 'L'])
+        assert_true(comparerecords(data, array))
